unit HiImageLoader; {    ImageLoader}

interface

uses Windows,Messages,Share,Debug,ActiveX,KOLComObj,KOL;

const
   IMGCHG_SIZE         = $0001;
   IMGCHG_VIEW         = $0002;
   IMGCHG_COMPLETE     = $0004;
   IMGCHG_ANIMATE      = $0008;
   IMGCHG_MASK         = $000F;

   IMGLOAD_NOTLOADED   = $00100000;
   IMGLOAD_LOADING     = $00200000;
   IMGLOAD_STOPPED     = $00400000;
   IMGLOAD_ERROR       = $00800000;
   IMGLOAD_COMPLETE    = $01000000;
   IMGLOAD_MASK        = $01F00000;

   IMGBITS_NONE        = $02000000;
   IMGBITS_PARTIAL     = $04000000;
   IMGBITS_TOTAL       = $08000000;
   IMGBITS_MASK        = $0E000000;

   IMGANIM_ANIMATED    = $10000000;
   IMGANIM_MASK        = $10000000;

   IMGTRANS_OPAQUE     = $20000000;
   IMGTRANS_MASK       = $20000000;

   DWN_COLORMODE       = $0000003F;
   DWN_DOWNLOADONLY    = $00000040;
   DWN_FORCEDITHER     = $00000080;
   DWN_RAWIMAGE        = $00000100;

type
   IImgCtx = interface(IUnknown)
      ['{3050f3d7-98b5-11cf-bb82-00aa00bdce0b}']
      // Initialization/Download methods
      function Load(pszUrl:PWChar; dwFlags:DWORD): HResult; stdcall;
      function SelectChanges(ulChgOn:DWORD; ulChgOff:DWORD; fSignal:DWORD): HResult; stdcall;
      function SetCallback(pCallback:pointer; pUserData:pointer): HResult; stdcall;
      function Disconnect: HResult; stdcall;

      // Query methods
      function GetUpdateRects(prc:PRect; var prcImg:TRect; var pcrc:integer): HResult; stdcall;
      function GetStateInfo(var pulState:DWORD; var pSize:TSize; fClearChanges:DWORD): HResult; stdcall;
      function GetPalette(var phpal:HPalette): HResult; stdcall;

      // Rendering methods
      function Draw(_hdc:HDC; var prcBounds:TRect): HResult; stdcall;
      function Tile(_hdc:HDC; var pptBackOrg:TPoint; var prcClip:TRect; var psize:TSize): HResult; stdcall;
      function StretchBlt(_hdc:HDC; dstX,dstY,dstXE,dstYE,srcX,srcY,srcXE,srcYE:integer; dwROP:DWORD): HResult; stdcall;
 end;

THiImageLoader = class(TDebug)
   private
      DC          : HDC;
      wsz         : integer;
      hsz         : integer;
      valid       : boolean;
      x1,x2,y1,y2 : integer;
   public
      FImgCtx              : IImgCtx;
      hOK                  : THandle;
      _prop_FileName       : string;
      _prop_DrawSource     : byte;
      _prop_Point2AsOffset : boolean;
      _data_FileName       : THI_Event;
      _data_Bitmap         : THI_Event;
      _data_Point1         : THI_Event;
      _data_Point2         : THI_Event;      
      _event_onLoad        : THI_Event;
      procedure _work_doLoad(var _Data:TData; idx:word);
end;
implementation

const
   CLSID_IImgCtx:TGUID = '{3050f3d6-98b5-11cf-bb82-00aa00bdce0b}';
   IID_IImgCtx:TGUID   = '{3050f3d7-98b5-11cf-bb82-00aa00bdce0b}';

{ StringToWideString }
function StringToWideString(const s: String; codePage: Word): WideString;
var   l: integer;
begin
   Result := '';
   if s = '' then exit;
   l := MultiByteToWideChar(codePage, MB_PRECOMPOSED, PChar(@s[1]), -1, nil, 0);
   SetLength(Result, l - 1);
   if l <= 1 then exit;
   MultiByteToWideChar(CodePage, MB_PRECOMPOSED, PChar(@s[1]), -1, PWideChar(@Result[1]), l - 1);
end;

procedure MyCallback(pCtx:pointer; pUserData:pointer); stdcall;
var   pCls:THiImageLoader; stat:DWORD; sz:TSize; bmp:PBitmap; Rect:TRect;
      hl,ht,hw,hh: integer;
begin
   pCls := THiImageLoader(pUserData);
   pCls.FImgCtx.GetStateInfo(stat, sz, 0);
   hl := pCls.x1;
   ht := pCls.y1;
   hw := pCls.x2 - hl;
   hh := pCls.y2 - ht;
   if  hw > (pCls.wsz - hl) then hw := pCls.wsz - hl;
   if  hh > (pCls.hsz - ht) then hh := pCls.hsz - ht;
   if (hw = 0) or (hh = 0) then begin
      hl := 0;
      ht := 0;
      hw := pCls.wsz;
      hh := pCls.hsz;
   end;
   if pCls.valid then begin
      if (sz.cx * 100) div sz.cy > (hw * 100) div hh then
         pCls.FImgCtx.StretchBlt(pCls.DC, hl, ht + (hh - (hw * sz.cy) div sz.cx) div 2, hw, (hw * sz.cy) div sz.cx, 0, 0, sz.cx, sz.cy, SRCCOPY)
      else
         pCls.FImgCtx.StretchBlt(pCls.DC, hl + (hw - (hh * sz.cx) div sz.cy) div 2, ht, (hh * sz.cx) div sz.cy, hh, 0, 0, sz.cx, sz.cy, SRCCOPY);
      _hi_OnEvent(pCls._event_onLoad);
   end else begin
      bmp := NewDIBBitmap(sz.cx, sz.cy, pf32bit);
      Rect := MakeRect(0, 0, sz.cx, sz.cy); 
      pCls.FImgCtx.Draw(bmp.Canvas.Handle, Rect);   
      _hi_OnEvent(pCls._event_onLoad, bmp);
      bmp.Free;
   end;
   SetEvent(pCls.hOK);
end;

procedure THiImageLoader._work_doLoad;
var   Bitmap: PBitmap; Wnd: HWND;
      wName: WideString; s: string; m: TMsg; hRect: TRect;
      p: dword;
begin
   hOK := CreateEvent(nil,false,false,nil);
   s := ReadString(_Data,_data_FileName,_prop_FileName);
   p := ReadInteger(_Data,_data_Point1); 

   y1 := p shr 16;
   x1 := p and $FFFF;
   p := ReadInteger(_Data,_data_Point2);
   y2 := p shr 16;
   x2 := p and $FFFF;
   if _prop_Point2AsOffset then begin
      inc(x2,x1);
      inc(y2,y1);
   end;
   if not FileExists(s) then exit;
   wName := StringToWideString(s,3);
   valid := false;
   Bitmap := nil;
   Wnd := 0;
   case _prop_DrawSource of
      0: begin
            Bitmap := ToBitmapEvent(_data_Bitmap);
            if (Bitmap <> nil) and (not Bitmap.Empty) then begin 
               DC := Bitmap.Canvas.Handle;
               hRect := MakeRect(0, 0, Bitmap.Width, Bitmap.Height);
               valid := true;
            end;
         end;
      1: begin
            Wnd := ToIntegerEvent(_data_Bitmap);
            if Wnd <> 0 then begin
               DC := GetWindowDC(Wnd);
               GetWindowRect(Wnd,hRect);
               valid := true;
            end;   
         end;
   end;      

   wsz := hRect.Right - hRect.Left;
   hsz := hRect.Bottom - hRect.Top;
   
   FImgCtx := CreateComObject(CLSID_IImgCtx) as IImgCtx;
   FImgCtx.Load(PWChar(wName), 0);
   FImgCtx.SetCallback(@MyCallback, pointer(Self));
   FImgCtx.SelectChanges(IMGCHG_COMPLETE,0,1);
   while WaitForSingleObject(hOK,0) <> WAIT_OBJECT_0 do begin
      if not GetMessage(m,0,0,0) then break;
      TranslateMessage(m);
      DispatchMessage(m);
   end;
   CloseHandle(hOK);
   if Wnd <> 0 then
      ReleaseDC(Wnd,DC)
   else if (Bitmap <> nil) and (not Bitmap.Empty) then
      DeleteDC(DC);
end;

initialization
  CoInitialize(nil);
finalization
  CoUninitialize;

end.